unit z80daisy;

{$ifdef fpc}{$mode delphi}{$H+}{$endif}

interface
type
  tipo_z80_daisy=record
    reset:procedure(n:byte);pascal;
    ack:function(n:byte):byte;pascal;
    reti:procedure(n:byte);pascal;
    state:function(n:byte):byte;pascal;
    device_num:byte;
  end;
const
  Z80_DAISY_INT=$01;		// interrupt request mask */
  Z80_DAISY_IEO=$02;		// interrupt disable mask (IEO) */
var
  z80daisy_llamadas:array[0..5] of tipo_z80_daisy;

procedure z80daisy_reset;
function z80daisy_ack:byte;
procedure z80daisy_reti;
function z80daisy_state:boolean;
procedure z80daisy_init;

implementation
procedure z80daisy_init;
var
  f:byte;
begin
  for f:=0 to 5 do begin
    z80daisy_llamadas[f].reset:=nil;
    z80daisy_llamadas[f].ack:=nil;
    z80daisy_llamadas[f].reti:=nil;
    z80daisy_llamadas[f].state:=nil;
  end;
end;

procedure z80daisy_reset;
var
  f:byte;
begin
  for f:=0 to 5 do if addr(z80daisy_llamadas[f].reset)<>nil then z80daisy_llamadas[f].reset(z80daisy_llamadas[f].device_num);
end;

function z80daisy_ack:byte;
var
  f,state:byte;
begin
  for f:=0 to 5 do begin
    if addr(z80daisy_llamadas[f].state)<>nil then begin
      state:=z80daisy_llamadas[f].state(z80daisy_llamadas[f].device_num);
      if (state and Z80_DAISY_INT)<>0 then begin
			  z80daisy_ack:=z80daisy_llamadas[f].ack(z80daisy_llamadas[f].device_num);
        exit;
      end;
    end;
  end;
	z80daisy_ack:=0;
end;

procedure z80daisy_reti;
var
  f,state:byte;
begin
// loop over all devices; dev[0] is highest priority */
	for f:=0 to 5 do begin
		if addr(z80daisy_llamadas[f].state)<>nil then begin
      state:=z80daisy_llamadas[f].state(z80daisy_llamadas[f].device_num);
		  //if this device is asserting the IEO line, that's the one we want */
		  if (state and Z80_DAISY_IEO)<>0 then begin
  			z80daisy_llamadas[f].reti(z80daisy_llamadas[f].device_num);
        exit;
      end;
    end;
  end;
end;

function z80daisy_state:boolean;
var
  f,state:byte;
begin
for f:=0 to 5 do begin
		if addr(z80daisy_llamadas[f].state)<>nil then begin
      state:=z80daisy_llamadas[f].state(z80daisy_llamadas[f].device_num);
		  // if this device is asserting the INT line, that's the one we want */
		  if (state and Z80_DAISY_INT)<>0 then begin
        z80daisy_state:=true;
        exit;
      end;
		  // if this device is asserting the IEO line, it blocks everyone else */
		  if (state and Z80_DAISY_IEO)<>0 then begin
        z80daisy_state:=false;
        exit;
      end;
	end;
end;
z80daisy_state:=false;
end;

end.
